# -*- coding: utf-8 -*-

# source: https://github.com/cucumber/gherkin/blob/master/lib/gherkin/i18n.json

TRANSLATIONS = {
    "en": {
        "name": u"English",
        "native": u"English",
        "feature": u"Feature|Business Need|Ability",
        "background": u"Background",
        "scenario": u"Scenario",
        "scenario_outline": u"Scenario Outline|Scenario Template",
        "examples": u"Examples|Scenarios",
        "given": u"Given",
        "when": u"When",
        "then": u"Then",
        "and": u"And",
        "but": u"But"
    },
    "af": {
        "name": u"Afrikaans",
        "native": u"Afrikaans",
        "feature": u"Funksie|Besigheid Behoefte|Vermoë",
        "background": u"Agtergrond",
        "scenario": u"Situasie",
        "scenario_outline": u"Situasie Uiteensetting",
        "examples": u"Voorbeelde",
        "given": u"Gegewe",
        "when": u"Wanneer",
        "then": u"Dan",
        "and": u"En",
        "but": u"Maar"
    },
    "ar": {
        "name": u"Arabic",
        "native": u"العربية",
        "feature": u"خاصية",
        "background": u"الخلفية",
        "scenario": u"سيناريو",
        "scenario_outline": u"سيناريو مخطط",
        "examples": u"امثلة",
        "given": u"بفرض",
        "when": u"متى|عندما",
        "then": u"اذاً|ثم",
        "and": u"و",
        "but": u"لكن"
    },
    "bm": {
        "name": u"Malay",
        "native": u"Bahasa Melayu",
        "feature": u"Fungsi",
        "background": u"Latar Belakang",
        "scenario": u"Senario|Situai|Keadaan",
        "scenario_outline": u"Template Senario|Template Situai|Template Keadaan|Menggariskan Senario",
        "examples": u"Contoh",
        "given": u"Diberi|Bagi",
        "when": u"Apabila",
        "then": u"Maka|Kemudian",
        "and": u"Dan",
        "but": u"Tetapi|Tapi"
    },
    "bg": {
        "name": u"Bulgarian",
        "native": u"български",
        "feature": u"Функционалност",
        "background": u"Предистория",
        "scenario": u"Сценарий",
        "scenario_outline": u"Рамка на сценарий",
        "examples": u"Примери",
        "given": u"Дадено",
        "when": u"Когато",
        "then": u"То",
        "and": u"И",
        "but": u"Но"
    },
    "ca": {
        "name": u"Catalan",
        "native": u"català",
        "background": u"Rerefons|Antecedents",
        "feature": u"Característica|Funcionalitat",
        "scenario": u"Escenari",
        "scenario_outline": u"Esquema de l'escenari",
        "examples": u"Exemples",
        "given": u"Donat|Donada|Atès|Atesa",
        "when": u"Quan",
        "then": u"Aleshores|Cal",
        "and": u"I",
        "but": u"Però"
    },
    "cy-GB": {
        "name": u"Welsh",
        "native": u"Cymraeg",
        "background": u"Cefndir",
        "feature": u"Arwedd",
        "scenario": u"Scenario",
        "scenario_outline": u"Scenario Amlinellol",
        "examples": u"Enghreifftiau",
        "given": u"Anrhegedig a",
        "when": u"Pryd",
        "then": u"Yna",
        "and": u"A",
        "but": u"Ond"
    },
    "cs": {
        "name": u"Czech",
        "native": u"Česky",
        "feature": u"Požadavek",
        "background": u"Pozadí|Kontext",
        "scenario": u"Scénář",
        "scenario_outline": u"Náčrt Scénáře|Osnova scénáře",
        "examples": u"Příklady",
        "given": u"Pokud|Za předpokladu",
        "when": u"Když",
        "then": u"Pak",
        "and": u"A také|A",
        "but": u"Ale"
    },
    "da": {
        "name": u"Danish",
        "native": u"dansk",
        "feature": u"Egenskab",
        "background": u"Baggrund",
        "scenario": u"Scenarie",
        "scenario_outline": u"Abstrakt Scenario",
        "examples": u"Eksempler",
        "given": u"Givet",
        "when": u"Når",
        "then": u"Så",
        "and": u"Og",
        "but": u"Men"
    },
    "de": {
        "name": u"German",
        "native": u"Deutsch",
        "feature": u"Funktionalität",
        "background": u"Grundlage",
        "scenario": u"Szenario",
        "scenario_outline": u"Szenariogrundriss",
        "examples": u"Beispiele",
        "given": u"Angenommen|Gegeben sei|Gegeben seien",
        "when": u"Wenn",
        "then": u"Dann",
        "and": u"Und",
        "but": u"Aber"
    },
    "el": {
        "name": u"Greek",
        "native": u"Ελληνικά",
        "feature": u"Δυνατότητα|Λειτουργία",
        "background": u"Υπόβαθρο",
        "scenario": u"Σενάριο",
        "scenario_outline": u"Περιγραφή Σεναρίου",
        "examples": u"Παραδείγματα|Σενάρια",
        "given": u"Δεδομένου",
        "when": u"Όταν",
        "then": u"Τότε",
        "and": u"Και",
        "but": u"Αλλά"
    },
    "en-au": {
        "name": u"Australian",
        "native": u"Australian",
        "feature": u"Pretty much",
        "background": u"First off",
        "scenario": u"Awww, look mate",
        "scenario_outline": u"Reckon it's like",
        "examples": u"You'll wanna",
        "given": u"Y'know",
        "when": u"It's just unbelievable",
        "then": u"But at the end of the day I reckon",
        "and": u"Too right",
        "but": u"Yeah nah"
    },
    "en-lol": {
        "name": u"LOLCAT",
        "native": u"LOLCAT",
        "feature": u"OH HAI",
        "background": u"B4",
        "scenario": u"MISHUN",
        "scenario_outline": u"MISHUN SRSLY",
        "examples": u"EXAMPLZ",
        "given": u"I CAN HAZ",
        "when": u"WEN",
        "then": u"DEN",
        "and": u"AN",
        "but": u"BUT"
    },
    "en-old": {
        "name": u"Old English",
        "native": u"Englisc",
        "feature": u"Hwaet|Hwæt",
        "background": u"Aer|Ær",
        "scenario": u"Swa",
        "scenario_outline": u"Swa hwaer swa|Swa hwær swa",
        "examples": u"Se the|Se þe|Se ðe",
        "given": u"Thurh|Þurh|Ðurh",
        "when": u"Tha|Þa|Ða",
        "then": u"Tha|Þa|Ða|Tha the|Þa þe|Ða ðe",
        "and": u"Ond|7",
        "but": u"Ac"
    },
    "en-pirate": {
        "name": u"Pirate",
        "native": u"Pirate",
        "feature": u"Ahoy matey!",
        "background": u"Yo-ho-ho",
        "scenario": u"Heave to",
        "scenario_outline": u"Shiver me timbers",
        "examples": u"Dead men tell no tales",
        "given": u"Gangway!",
        "when": u"Blimey!",
        "then": u"Let go and haul",
        "and": u"Aye",
        "but": u"Avast!"
    },
    "en-Scouse": {
        "name": u"Scouse",
        "native": u"Scouse",
        "feature": u"Feature",
        "background": u"Dis is what went down",
        "scenario": u"The thing of it is",
        "scenario_outline": u"Wharrimean is",
        "examples": u"Examples",
        "given": u"Givun|Youse know when youse got",
        "when": u"Wun|Youse know like when",
        "then": u"Dun|Den youse gotta",
        "and": u"An",
        "but": u"Buh"
    },
    "en-tx": {
        "name": u"Texan",
        "native": u"Texan",
        "feature": u"Feature",
        "background": u"Background",
        "scenario": u"Scenario",
        "scenario_outline": u"All y'all",
        "examples": u"Examples",
        "given": u"Given y'all",
        "when": u"When y'all",
        "then": u"Then y'all",
        "and": u"And y'all",
        "but": u"But y'all"
    },
    "eo": {
        "name": u"Esperanto",
        "native": u"Esperanto",
        "feature": u"Trajto",
        "background": u"Fono",
        "scenario": u"Scenaro",
        "scenario_outline": u"Konturo de la scenaro",
        "examples": u"Ekzemploj",
        "given": u"Donitaĵo",
        "when": u"Se",
        "then": u"Do",
        "and": u"Kaj",
        "but": u"Sed"
    },
    "es": {
        "name": u"Spanish",
        "native": u"español",
        "background": u"Antecedentes",
        "feature": u"Característica",
        "scenario": u"Escenario",
        "scenario_outline": u"Esquema del escenario",
        "examples": u"Ejemplos",
        "given": u"Dado|Dada|Dados|Dadas",
        "when": u"Cuando",
        "then": u"Entonces",
        "and": u"Y",
        "but": u"Pero"
    },
    "et": {
        "name": u"Estonian",
        "native": u"eesti keel",
        "feature": u"Omadus",
        "background": u"Taust",
        "scenario": u"Stsenaarium",
        "scenario_outline": u"Raamstsenaarium",
        "examples": u"Juhtumid",
        "given": u"Eeldades",
        "when": u"Kui",
        "then": u"Siis",
        "and": u"Ja",
        "but": u"Kuid"
    },
    "fa": {
        "name": u"Persian",
        "native": u"فارسی",
        "feature": u"وِیژگی",
        "background": u"زمینه",
        "scenario": u"سناریو",
        "scenario_outline": u"الگوی سناریو",
        "examples": u"نمونه ها",
        "given": u"با فرض",
        "when": u"هنگامی",
        "then": u"آنگاه",
        "and": u"و",
        "but": u"اما"
    },
    "fi": {
        "name": u"Finnish",
        "native": u"suomi",
        "feature": u"Ominaisuus",
        "background": u"Tausta",
        "scenario": u"Tapaus",
        "scenario_outline": u"Tapausaihio",
        "examples": u"Tapaukset",
        "given": u"Oletetaan",
        "when": u"Kun",
        "then": u"Niin",
        "and": u"Ja",
        "but": u"Mutta"
    },
    "fr": {
        "name": u"French",
        "native": u"français",
        "feature": u"Fonctionnalité",
        "background": u"Contexte",
        "scenario": u"Scénario",
        "scenario_outline": u"Plan du scénario|Plan du Scénario",
        "examples": u"Exemples",
        "given": u"Soit|Etant donné|Etant donnée|Etant donnés|Etant données|Étant donné|Étant donnée|Étant donnés|Étant données",
        "when": u"Quand|Lorsque|Lorsqu'<",
        "then": u"Alors",
        "and": u"Et",
        "but": u"Mais"
    },
    "gl": {
        "name": u"Galician",
        "native": u"galego",
        "background": u"Contexto",
        "feature": u"Característica",
        "scenario": u"Escenario",
        "scenario_outline": u"Esbozo do escenario",
        "examples": u"Exemplos",
        "given": u"Dado|Dada|Dados|Dadas",
        "when": u"Cando",
        "then": u"Entón|Logo",
        "and": u"E",
        "but": u"Mais|Pero"
    },
    "he": {
        "name": u"Hebrew",
        "native": u"עברית",
        "feature": u"תכונה",
        "background": u"רקע",
        "scenario": u"תרחיש",
        "scenario_outline": u"תבנית תרחיש",
        "examples": u"דוגמאות",
        "given": u"בהינתן",
        "when": u"כאשר",
        "then": u"אז|אזי",
        "and": u"וגם",
        "but": u"אבל"
    },
    "hi": {
        "name": u"Hindi",
        "native": u"हिंदी",
        "feature": u"रूप लेख",
        "background": u"पृष्ठभूमि",
        "scenario": u"परिदृश्य",
        "scenario_outline": u"परिदृश्य रूपरेखा",
        "examples": u"उदाहरण",
        "given": u"अगर|यदि|चूंकि",
        "when": u"जब|कदा",
        "then": u"तब|तदा",
        "and": u"और|तथा",
        "but": u"पर|परन्तु|किन्तु"
    },
    "hr": {
        "name": u"Croatian",
        "native": u"hrvatski",
        "feature": u"Osobina|Mogućnost|Mogucnost",
        "background": u"Pozadina",
        "scenario": u"Scenarij",
        "scenario_outline": u"Skica|Koncept",
        "examples": u"Primjeri|Scenariji",
        "given": u"Zadan|Zadani|Zadano",
        "when": u"Kada|Kad",
        "then": u"Onda",
        "and": u"I",
        "but": u"Ali"
    },
    "ht": {
        "name": u"Creole",
        "native": u"kreyòl",
        "feature": u"Karakteristik|Mak|Fonksyonalite",
        "background": u"Kontèks|Istorik",
        "scenario": u"Senaryo",
        "scenario_outline": u"Plan senaryo|Plan Senaryo|Senaryo deskripsyon|Senaryo Deskripsyon|Dyagram senaryo|Dyagram Senaryo",
        "examples": u"Egzanp",
        "given": u"Sipoze|Sipoze ke|Sipoze Ke",
        "when": u"Lè|Le",
        "then": u"Lè sa a|Le sa a",
        "and": u"Ak|Epi|E",
        "but": u"Men"
    },
    "hu": {
        "name": u"Hungarian",
        "native": u"magyar",
        "feature": u"Jellemző",
        "background": u"Háttér",
        "scenario": u"Forgatókönyv",
        "scenario_outline": u"Forgatókönyv vázlat",
        "examples": u"Példák",
        "given": u"Amennyiben|Adott",
        "when": u"Majd|Ha|Amikor",
        "then": u"Akkor",
        "and": u"És",
        "but": u"De"
    },
    "id": {
        "name": u"Indonesian",
        "native": u"Bahasa Indonesia",
        "feature": u"Fitur",
        "background": u"Dasar",
        "scenario": u"Skenario",
        "scenario_outline": u"Skenario konsep",
        "examples": u"Contoh",
        "given": u"Dengan",
        "when": u"Ketika",
        "then": u"Maka",
        "and": u"Dan",
        "but": u"Tapi"
    },
    "is": {
        "name": u"Icelandic",
        "native": u"Íslenska",
        "feature": u"Eiginleiki",
        "background": u"Bakgrunnur",
        "scenario": u"Atburðarás",
        "scenario_outline": u"Lýsing Atburðarásar|Lýsing Dæma",
        "examples": u"Dæmi|Atburðarásir",
        "given": u"Ef",
        "when": u"Þegar",
        "then": u"Þá",
        "and": u"Og",
        "but": u"En"
    },
    "it": {
        "name": u"Italian",
        "native": u"italiano",
        "feature": u"Funzionalità",
        "background": u"Contesto",
        "scenario": u"Scenario",
        "scenario_outline": u"Schema dello scenario",
        "examples": u"Esempi",
        "given": u"Dato|Data|Dati|Date",
        "when": u"Quando",
        "then": u"Allora",
        "and": u"E",
        "but": u"Ma"
    },
    "ja": {
        "name": u"Japanese",
        "native": u"日本語",
        "feature": u"フィーチャ|機能",
        "background": u"背景",
        "scenario": u"シナリオ",
        "scenario_outline": u"シナリオアウトライン|シナリオテンプレート|テンプレ|シナリオテンプレ",
        "examples": u"例|サンプル",
        "given": u"前提<",
        "when": u"もし<",
        "then": u"ならば<",
        "and": u"かつ<",
        "but": u"しかし<|但し<|ただし<"
    },
    "jv": {
        "name": u"Javanese",
        "native": u"Basa Jawa",
        "feature": u"Fitur",
        "background": u"Dasar",
        "scenario": u"Skenario",
        "scenario_outline": u"Konsep skenario",
        "examples": u"Conto|Contone",
        "given": u"Nalika|Nalikaning",
        "when": u"Manawa|Menawa",
        "then": u"Njuk|Banjur",
        "and": u"Lan",
        "but": u"Tapi|Nanging|Ananging"
    },
    "kn": {
        "name": u"Kannada",
        "native": u"ಕನ್ನಡ",
        "background": u"ಹಿನ್ನೆಲೆ",
        "feature": u"ಹೆಚ್ಚಳ",
        "scenario": u"ಕಥಾಸಾರಾಂಶ",
        "scenario_outline": u"ವಿವರಣೆ",
        "examples": u"ಉದಾಹರಣೆಗಳು",
        "given": u"ನೀಡಿದ",
        "when": u"ಸ್ಥಿತಿಯನ್ನು",
        "then": u"ನಂತರ",
        "and": u"ಮತ್ತು",
        "but": u"ಆದರೆ"
    },
    "ko": {
        "name": u"Korean",
        "native": u"한국어",
        "background": u"배경",
        "feature": u"기능",
        "scenario": u"시나리오",
        "scenario_outline": u"시나리오 개요",
        "examples": u"예",
        "given": u"조건<|먼저<",
        "when": u"만일<|만약<",
        "then": u"그러면<",
        "and": u"그리고<",
        "but": u"하지만<|단<"
    },
    "lt": {
        "name": u"Lithuanian",
        "native": u"lietuvių kalba",
        "feature": u"Savybė",
        "background": u"Kontekstas",
        "scenario": u"Scenarijus",
        "scenario_outline": u"Scenarijaus šablonas",
        "examples": u"Pavyzdžiai|Scenarijai|Variantai",
        "given": u"Duota",
        "when": u"Kai",
        "then": u"Tada",
        "and": u"Ir",
        "but": u"Bet"
    },
    "lu": {
        "name": u"Luxemburgish",
        "native": u"Lëtzebuergesch",
        "feature": u"Funktionalitéit",
        "background": u"Hannergrond",
        "scenario": u"Szenario",
        "scenario_outline": u"Plang vum Szenario",
        "examples": u"Beispiller",
        "given": u"ugeholl",
        "when": u"wann",
        "then": u"dann",
        "and": u"an|a",
        "but": u"awer|mä"
    },
    "lv": {
        "name": u"Latvian",
        "native": u"latviešu",
        "feature": u"Funkcionalitāte|Fīča",
        "background": u"Konteksts|Situācija",
        "scenario": u"Scenārijs",
        "scenario_outline": u"Scenārijs pēc parauga",
        "examples": u"Piemēri|Paraugs",
        "given": u"Kad",
        "when": u"Ja",
        "then": u"Tad",
        "and": u"Un",
        "but": u"Bet"
    },
    "nl": {
        "name": u"Dutch",
        "native": u"Nederlands",
        "feature": u"Functionaliteit",
        "background": u"Achtergrond",
        "scenario": u"Scenario",
        "scenario_outline": u"Abstract Scenario",
        "examples": u"Voorbeelden",
        "given": u"Gegeven|Stel",
        "when": u"Als",
        "then": u"Dan",
        "and": u"En",
        "but": u"Maar"
    },
    "no": {
        "name": u"Norwegian",
        "native": u"norsk",
        "feature": u"Egenskap",
        "background": u"Bakgrunn",
        "scenario": u"Scenario",
        "scenario_outline": u"Scenariomal|Abstrakt Scenario",
        "examples": u"Eksempler",
        "given": u"Gitt",
        "when": u"Når",
        "then": u"Så",
        "and": u"Og",
        "but": u"Men"
    },
    "pa": {
        "name": u"Panjabi",
        "native": u"ਪੰਜਾਬੀ",
        "feature": u"ਖਾਸੀਅਤ|ਮੁਹਾਂਦਰਾ|ਨਕਸ਼ ਨੁਹਾਰ",
        "background": u"ਪਿਛੋਕੜ",
        "scenario": u"ਪਟਕਥਾ",
        "scenario_outline": u"ਪਟਕਥਾ ਢਾਂਚਾ|ਪਟਕਥਾ ਰੂਪ ਰੇਖਾ",
        "examples": u"ਉਦਾਹਰਨਾਂ",
        "given": u"ਜੇਕਰ|ਜਿਵੇਂ ਕਿ",
        "when": u"ਜਦੋਂ",
        "then": u"ਤਦ",
        "and": u"ਅਤੇ",
        "but": u"ਪਰ"
    },
    "pl": {
        "name": u"Polish",
        "native": u"polski",
        "feature": u"Właściwość|Funkcja|Aspekt|Potrzeba biznesowa",
        "background": u"Założenia",
        "scenario": u"Scenariusz",
        "scenario_outline": u"Szablon scenariusza",
        "examples": u"Przykłady",
        "given": u"Zakładając(?:, że)?|Mając",
        "when": u"Jeżeli|Jeśli|Gdy|Kiedy",
        "then": u"Wtedy",
        "and": u"Oraz|I",
        "but": u"Ale"
    },
    "pt": {
        "name": u"Portuguese",
        "native": u"português",
        "background": u"Contexto|Cenário de Fundo|Cenario de Fundo|Fundo",
        "feature": u"Funcionalidade|Característica|Caracteristica",
        "scenario": u"Cenário|Cenario",
        "scenario_outline": u"Esquema do Cenário|Esquema do Cenario|Delineação do Cenário|Delineacao do Cenario",
        "examples": u"Exemplos|Cenários|Cenarios",
        "given": u"Dado|Dada|Dados|Dadas",
        "when": u"Quando",
        "then": u"Então|Entao",
        "and": u"E",
        "but": u"Mas"
    },
    "ro": {
        "name": u"Romanian",
        "native": u"română",
        "background": u"Context",
        "feature": u"Functionalitate|Funcționalitate|Funcţionalitate",
        "scenario": u"Scenariu",
        "scenario_outline": u"Structura scenariu|Structură scenariu",
        "examples": u"Exemple",
        "given": u"Date fiind|Dat fiind|Dati fiind|Dați fiind|Daţi fiind",
        "when": u"Cand|Când",
        "then": u"Atunci",
        "and": u"Si|Și|Şi",
        "but": u"Dar"
    },
    "ru": {
        "name": u"Russian",
        "native": u"русский",
        "feature": u"Функция|Функционал|Свойство",
        "background": u"Предыстория|Контекст",
        "scenario": u"Сценарий",
        "scenario_outline": u"Структура сценария",
        "examples": u"Примеры",
        "given": u"Допустим|Дано|Пусть",
        "when": u"Если|Когда",
        "then": u"То|Тогда",
        "and": u"И|К тому же|Также",
        "but": u"Но|А"
    },
    "sv": {
        "name": u"Swedish",
        "native": u"Svenska",
        "feature": u"Egenskap",
        "background": u"Bakgrund",
        "scenario": u"Scenario",
        "scenario_outline": u"Abstrakt Scenario|Scenariomall",
        "examples": u"Exempel",
        "given": u"Givet",
        "when": u"När",
        "then": u"Så",
        "and": u"Och",
        "but": u"Men"
    },
    "sk": {
        "name": u"Slovak",
        "native": u"Slovensky",
        "feature": u"Požiadavka|Funkcia|Vlastnosť",
        "background": u"Pozadie",
        "scenario": u"Scenár",
        "scenario_outline": u"Náčrt Scenáru|Náčrt Scenára|Osnova Scenára",
        "examples": u"Príklady",
        "given": u"Pokiaľ|Za predpokladu",
        "when": u"Keď|Ak",
        "then": u"Tak|Potom",
        "and": u"A|A tiež|A taktiež|A zároveň",
        "but": u"Ale"
    },
    "sl": {
        "name": u"Slovenian",
        "native": u"Slovenski",
        "feature": u"Funkcionalnost|Funkcija|Možnosti|Moznosti|Lastnost|Značilnost",
        "background": u"Kontekst|Osnova|Ozadje",
        "scenario": u"Scenarij|Primer",
        "scenario_outline": u"Struktura scenarija|Skica|Koncept|Oris scenarija|Osnutek",
        "examples": u"Primeri|Scenariji",
        "given": u"Dano|Podano|Zaradi|Privzeto",
        "when": u"Ko|Ce|Če|Kadar",
        "then": u"Nato|Potem|Takrat",
        "and": u"In|Ter",
        "but": u"Toda|Ampak|Vendar"
    },
    "sr-Latn": {
        "name": u"Serbian (Latin)",
        "native": u"Srpski (Latinica)",
        "feature": u"Funkcionalnost|Mogućnost|Mogucnost|Osobina",
        "background": u"Kontekst|Osnova|Pozadina",
        "scenario": u"Scenario|Primer",
        "scenario_outline": u"Struktura scenarija|Skica|Koncept",
        "examples": u"Primeri|Scenariji",
        "given": u"Zadato|Zadate|Zatati",
        "when": u"Kada|Kad",
        "then": u"Onda",
        "and": u"I",
        "but": u"Ali"
    },
    "sr-Cyrl": {
        "name": u"Serbian",
        "native": u"Српски",
        "feature": u"Функционалност|Могућност|Особина",
        "background": u"Контекст|Основа|Позадина",
        "scenario": u"Сценарио|Пример",
        "scenario_outline": u"Структура сценарија|Скица|Концепт",
        "examples": u"Примери|Сценарији",
        "given": u"Задато|Задате|Задати",
        "when": u"Када|Кад",
        "then": u"Онда",
        "and": u"И",
        "but": u"Али"
    },
    "tl": {
        "name": u"Telugu",
        "native": u"తెలుగు",
        "feature": u"గుణము",
        "background": u"నేపథ్యం",
        "scenario": u"సన్నివేశం",
        "scenario_outline": u"కథనం",
        "examples": u"ఉదాహరణలు",
        "given": u"చెప్పబడినది",
        "when": u"ఈ పరిస్థితిలో",
        "then": u"అప్పుడు",
        "and": u"మరియు",
        "but": u"కాని"
    },
    "th": {
        "name": u"Thai",
        "native": u"ไทย",
        "feature": u"โครงหลัก|ความต้องการทางธุรกิจ|ความสามารถ",
        "background": u"แนวคิด",
        "scenario": u"เหตุการณ์",
        "scenario_outline": u"สรุปเหตุการณ์|โครงสร้างของเหตุการณ์",
        "examples": u"ชุดของตัวอย่าง|ชุดของเหตุการณ์",
        "given": u"กำหนดให้",
        "when": u"เมื่อ",
        "then": u"ดังนั้น",
        "and": u"และ",
        "but": u"แต่"
    },
    "tlh": {
        "name": u"Klingon",
        "native": u"tlhIngan",
        "feature": u"Qap|Qu'meH 'ut|perbogh|poQbogh malja'|laH",
        "background": u"mo'",
        "scenario": u"lut",
        "scenario_outline": u"lut chovnatlh",
        "examples": u"ghantoH|lutmey",
        "given": u"ghu' noblu'|DaH ghu' bejlu'",
        "when": u"qaSDI'",
        "then": u"vaj",
        "and": u"'ej|latlh",
        "but": u"'ach|'a"
    },
    "tr": {
        "name": u"Turkish",
        "native": u"Türkçe",
        "feature": u"Özellik",
        "background": u"Geçmiş",
        "scenario": u"Senaryo",
        "scenario_outline": u"Senaryo taslağı",
        "examples": u"Örnekler",
        "given": u"Diyelim ki",
        "when": u"Eğer ki",
        "then": u"O zaman",
        "and": u"Ve",
        "but": u"Fakat|Ama"
    },
    "tt": {
        "name": u"Tatar",
        "native": u"Татарча",
        "feature": u"Мөмкинлек|Үзенчәлеклелек",
        "background": u"Кереш",
        "scenario": u"Сценарий",
        "scenario_outline": u"Сценарийның төзелеше",
        "examples": u"Үрнәкләр|Мисаллар",
        "given": u"Әйтик",
        "when": u"Әгәр",
        "then": u"Нәтиҗәдә",
        "and": u"Һәм|Вә",
        "but": u"Ләкин|Әмма"
    },
    "uk": {
        "name": u"Ukrainian",
        "native": u"Українська",
        "feature": u"Функціонал",
        "background": u"Передумова",
        "scenario": u"Сценарій",
        "scenario_outline": u"Структура сценарію",
        "examples": u"Приклади",
        "given": u"Припустимо|Припустимо, що|Нехай|Дано",
        "when": u"Якщо|Коли",
        "then": u"То|Тоді",
        "and": u"І|А також|Та",
        "but": u"Але"
    },
    "uz": {
        "name": u"Uzbek",
        "native": u"Узбекча",
        "feature": u"Функционал",
        "background": u"Тарих",
        "scenario": u"Сценарий",
        "scenario_outline": u"Сценарий структураси",
        "examples": u"Мисоллар",
        "given": u"Агар",
        "when": u"Агар",
        "then": u"Унда",
        "and": u"Ва",
        "but": u"Лекин|Бирок|Аммо"
    },
    "vi": {
        "name": u"Vietnamese",
        "native": u"Tiếng Việt",
        "feature": u"Tính năng",
        "background": u"Bối cảnh",
        "scenario": u"Tình huống|Kịch bản",
        "scenario_outline": u"Khung tình huống|Khung kịch bản",
        "examples": u"Dữ liệu",
        "given": u"Biết|Cho",
        "when": u"Khi",
        "then": u"Thì",
        "and": u"Và",
        "but": u"Nhưng"
    },
    "zh-CN": {
        "name": u"Chinese simplified",
        "native": u"简体中文",
        "feature": u"功能",
        "background": u"背景",
        "scenario": u"场景|剧本",
        "scenario_outline": u"场景大纲|剧本大纲",
        "examples": u"例子",
        "given": u"假如<|假设<|假定<",
        "when": u"当<",
        "then": u"那么<",
        "and": u"而且<|并且<|同时<",
        "but": u"但是<"
    },
    "zh-TW": {
        "name": u"Chinese traditional",
        "native": u"繁體中文",
        "feature": u"功能",
        "background": u"背景",
        "scenario": u"場景|劇本",
        "scenario_outline": u"場景大綱|劇本大綱",
        "examples": u"例子",
        "given": u"假如<|假設<|假定<",
        "when": u"當<",
        "then": u"那麼<",
        "and": u"而且<|並且<|同時<",
        "but": u"但是<"
    },
    "ur": {
        "name": u"Urdu",
        "native": u"اردو",
        "feature": u"صلاحیت|کاروبار کی ضرورت|خصوصیت",
        "background": u"پس منظر",
        "scenario": u"منظرنامہ",
        "scenario_outline": u"منظر نامے کا خاکہ",
        "examples": u"مثالیں",
        "given": u"اگر|بالفرض|فرض کیا",
        "when": u"جب",
        "then": u"پھر|تب",
        "and": u"اور",
        "but": u"لیکن"
    }
}
